# -*- Mode: Python -*-
# -*- coding: ascii -*-

"""
This is a script to convert the points of polygons to curves.
"""

__author__      = 'Ernest Chan'
__date__        = 'November 2015'
__copyright__   = 'Copyright (C) 2015'
__version__     = '1.0'
__maintainer__  = 'Ernest Chan'
__email__       = 'ernpchan@gmail.com'
__status__      = 'Publish'
__lwver__       = '11'

import lwsdk
reload(lwsdk)

lwMF = lwsdk.LWMessageFuncs()
ui = lwsdk.LWPanels()

from lwsdk import ModCommand
from lwsdk.pris.modeler import init, editbegin, editend, getpolygons, polyinfo, addcurve, undogroupbegin, undogroupend, moninit, monstep, monend, lyrfg, lyrsetfg, smoothcurves, selpolygon, selmode

init(ModCommand())

# lscript documentation
# http://www.mikegreen.name/Lscript/Lscript%20Index.html

# python and sdk documentation
# https://www.lightwave3d.com/lightwave_sdk/

class CurveFromPoly(lwsdk.ICommandSequence):
    def __init__(self, context):
        super(CurveFromPoly, self).__init__()
        
    # LWGenericHandler ----------------------------------
    def process(self, mod_command):      
        # begin mesh-edit operation
        editbegin()
        
        # get polygon(s) of current layer(s)
        polygons = getpolygons()
        
        # check if there are polygons 
        # if there are no polygons, error out
        if len(polygons) == 0:
            # end mesh-edit operation
            editend()
            # error message
            lwMF.error('','No polygon(s).')
            return lwsdk.AFUNC_OK

        # create ui
        panel = ui.create('CurveFromPoly')
        
        # feedback item
        feedback1 = panel.text_ctl('',['Having nothing selected means all the polygons will generate curves.'])
        
        # string ui item
        self.surface = panel.str_ctl('Curve Surface Name:',30)
        # set ui item to specific value
        # change the argument to make ui launch with a default value
        self.surface.set_str('')
        
        # feedback item
        feedback2 = panel.text_ctl('',['An empty string will create a default surface name of \'Curve.\''])
        
        # user option to create an open or closed curve
        self.curveChoice = panel.hchoice_ctl('Make Curve:',['Open','Closed'])

        feedback3 = panel.text_ctl('',['If you want to undo the operation after doing this on multiple layers,','you have to run undo on each layer individually.','Creating closed curves will mean you have to undo each curve.','Otherwise, you will be able to undo the operation with a single undo.'])
        
        # user hit 'Cancel'
        if panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
            ui.destroy(panel)
            
            # end mesh-edit operation
            editend()
            
            return lwsdk.AFUNC_OK
            
        else:    
            # close out original mesh-edit operation
            editend()
            # begin undogroup wrapper
            undogroupbegin
            # initialize monitor
            moninit(len(polygons),'Making Curves...')
            # get fg layers
            myLayers = lyrfg()
            
            # print self.curveChoice.get_int()
            
            # iterate through layers
            for i in myLayers:
                # set active fg layer
                lyrsetfg(i)
                # start mesh-edit operation
                editbegin()
                # get polygon(s) of layer
                lyrPolygons = getpolygons()
                # check if layer has polygons
                if len(lyrPolygons) > 0:        
                    # begin iteration through polygons
                    for i in lyrPolygons:
                        # get information about polygon
                        # polyinfo returns the surface name of the polygon and all the points that make it
                        pi = polyinfo(i)
                        
                        # polyinfo returns surface name as first item so we have to pop (remove) it
                        pi.pop(0)
                        
                        # check if user wants to make a closed curve
                        if self.curveChoice.get_int() == 1:
                            # Per lscript documentation on addcurve 'To create a closed curve, the beginning and ending points provided must overlap.' so we need to add the first point of the curve to the end of our list of points.
                            # This information is in the lscript dox but not the python dox.
                            pi.append(pi[0])
                        # user entered an empty string -> default to Curve
                        if self.surface.get_str() is '':
                            newCurve = 'Curve'
                        else:
                            # get user's custom string
                            newCurve = self.surface.get_str()
                        
                        # call addcurve command
                        # arguments are points and optional surface name
                        # addcurve command returns id of the curve it makes so we should grab it 
                        # because we'll need it if the user wants to make a closed curve
                        myCurve = addcurve(pi,newCurve)
                        
                        # check if user wants to make a closed curve
                        if self.curveChoice.get_int() == 1:
                            editend()
                            selmode(lwsdk.OPSEL_USER)
                            # select the curve that was created from the addcurve command
                            selpolygon('SET','POLYID',myCurve)
                            editbegin()
                            selmode(lwsdk.OPSEL_DIRECT)
                            # call smooth curves command to properly smooth the curve between the first and last point
                            smoothcurves()
                            editend()
                            # clear selection
                            selpolygon('CLEAR','CURVE')
                            editbegin()
                        # monitor progress increment
                        monstep()
                        
                # end mesh-edit operation
                editend()
            
            # end monitor
            monend()
                
            # end undogroup wrapper
            undogroupend
            
            # restore user's original fg layers
            lyrsetfg(' '.join(myLayers))
            
            # notify user script is done
            lwMF.info('','Done.')
            
            return lwsdk.AFUNC_OK
    
ServerTagInfo = [
                    ( 'CurveFromPoly',
                    lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                    ( 'CurveFromPoly',
                    lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH ),
                ]
 
ServerRecord = { lwsdk.CommandSequenceFactory('CurveFromPoly', CurveFromPoly) : ServerTagInfo }